@@ -107,7 +107,7 @@ def _expand_file_prefixed_files(args):
         except IndexError:
             return _maybe_load_file(arg_split[0])
 
-    return list([_expand_file_prefix(arg) for arg in args])
+    return [_expand_file_prefix(arg) for arg in args]
 
 
 def _pre_command_table_create(cli_ctx, args):
@@ -304,7 +304,7 @@ class AzCliCommand(CLICommand):
                 local_context = self.cli_ctx.local_context
                 value = local_context.get(self.name, lca.name)
                 if value:
-                    logger.debug("local context '%s' for arg %s", value, arg.name)
+                    logger.debug("parameter persistence '%s' for arg %s", value, arg.name)
                     overrides.settings['default'] = DefaultStr(value)
                     overrides.settings['required'] = False
                     overrides.settings['default_value_source'] = 'Local Context'
@@ -596,13 +596,13 @@ class AzCliCommandInvoker(CommandInvoker):
                     value = getattr(parsed_args, name)
                     local_context_args.append((options[0], value))
             if local_context_args:
-                logger.warning('Local context is turned on. Its information is saved in working directory %s. You can '
-                               'run `az local-context off` to turn it off.',
+                logger.warning('Parameter persistence is turned on. Its information is saved in working directory %s. '
+                               'You can run `az config param-persist off` to turn it off.',
                                self.cli_ctx.local_context.effective_working_directory())
                 args_str = []
                 for name, value in local_context_args:
                     args_str.append('{}: {}'.format(name, value))
-                logger.warning('Command argument values from local context: %s', ', '.join(args_str))
+                logger.warning('Command argument values from persistent parameters: %s', ', '.join(args_str))
 
         # TODO: This fundamentally alters the way Knack.invocation works here. Cannot be customized
         # with an event. Would need to be customized via inheritance.
@@ -875,7 +875,11 @@ class LongRunningOperation:  # pylint: disable=too-few-public-methods
         self.poller_done_interval_ms = poller_done_interval_ms
         self.deploy_dict = {}
         self.last_progress_report = datetime.datetime.now()
-        self.progress_bar = progress_bar if progress_bar is not None else IndeterminateProgressBar(cli_ctx)
+
+        self.progress_bar = None
+        disable_progress_bar = self.cli_ctx.config.getboolean('core', 'disable_progress_bar', False)
+        if not disable_progress_bar and not cli_ctx.only_show_errors:
+            self.progress_bar = progress_bar if progress_bar is not None else IndeterminateProgressBar(cli_ctx)
 
     def _delay(self):
         time.sleep(self.poller_done_interval_ms / 1000.0)
@@ -943,11 +947,13 @@ class LongRunningOperation:  # pylint: disable=too-few-public-methods
                             if update:
                                 logger.info(result)
 
-    def __call__(self, poller):
+    def __call__(self, poller):  # pylint: disable=too-many-statements
         from msrest.exceptions import ClientException
+        from azure.core.exceptions import HttpResponseError
 
         correlation_message = ''
-        self.progress_bar.begin()
+        if self.progress_bar:
+            self.progress_bar.begin()
         correlation_id = None
 
         cli_logger = get_logger()  # get CLI logger which has the level set through command lines
@@ -975,23 +981,35 @@ class LongRunningOperation:  # pylint: disable=too-few-public-methods
                 except Exception as ex:  # pylint: disable=broad-except
                     logger.warning('%s during progress reporting: %s', getattr(type(ex), '__name__', type(ex)), ex)
             try:
-                self.progress_bar.update_progress()
+                if self.progress_bar:
+                    self.progress_bar.update_progress()
                 self._delay()
             except KeyboardInterrupt:
-                self.progress_bar.stop()
+                if self.progress_bar:
+                    self.progress_bar.stop()
                 logger.error('Long-running operation wait cancelled.  %s', correlation_message)
                 raise
 
         try:
             result = poller.result()
-        except ClientException as client_exception:
+        except (ClientException, HttpResponseError) as exception:
             from azure.cli.core.commands.arm import handle_long_running_operation_exception
-            self.progress_bar.stop()
-            handle_long_running_operation_exception(client_exception)
-
-        self.progress_bar.end()
-        if poll_flag:
-            telemetry.poll_end()
+            if self.progress_bar:
+                self.progress_bar.stop()
+            if getattr(exception, 'status_code', None) == 404 and \
+               ('delete' in self.cli_ctx.data['command'] or 'purge' in self.cli_ctx.data['command']):
+                logger.debug('Service returned 404 on the long-running delete or purge operation. CLI treats it as '
+                             'delete or purge successfully but service should fix this behavior.')
+                return None
+            if isinstance(exception, ClientException):
+                handle_long_running_operation_exception(exception)
+            else:
+                raise exception
+        finally:
+            if self.progress_bar:
+                self.progress_bar.end()
+            if poll_flag:
+                telemetry.poll_end()
 
         return result
 
@@ -1115,8 +1133,8 @@ def _load_azure_exception_class():
 # pylint: disable=no-member
 def _is_paged(obj):
     # Since loading msrest is expensive, we avoid it until we have to
-    import collections
-    if isinstance(obj, collections.Iterable) \
+    from collections.abc import Iterable
+    if isinstance(obj, Iterable) \
             and not isinstance(obj, list) \
             and not isinstance(obj, dict):
         from msrest.paging import Paged
@@ -1162,6 +1180,12 @@ class CliCommandType:
 class AzCommandGroup(CommandGroup):
 
     def __init__(self, command_loader, group_name, **kwargs):
+        """
+        :param command_loader: The command loader that commands will be registered into
+        :type command_loader: azure.cli.core.AzCommandsLoader
+        :param group_name: The name of the group of commands in the command hierarchy
+        :type group_name: str
+        """
         merged_kwargs = self._merge_kwargs(kwargs, base_kwargs=command_loader.module_kwargs)
         operations_tmpl = merged_kwargs.pop('operations_tmpl', None)
         super(AzCommandGroup, self).__init__(command_loader, group_name,
@@ -1249,16 +1273,24 @@ class AzCommandGroup(CommandGroup):
         return self._command(name, method_name=method_name, custom_command=True, **kwargs)
 
     def _command(self, name, method_name, custom_command=False, **kwargs):
+        from .command_operation import CommandOperation
+
         self._check_stale()
         merged_kwargs = self._flatten_kwargs(kwargs, get_command_type_kwarg(custom_command))
         self._apply_tags(merged_kwargs, kwargs, name)
 
         operations_tmpl = merged_kwargs['operations_tmpl']
-        command_name = '{} {}'.format(self.group_name, name) if self.group_name else name
-        self.command_loader._cli_command(command_name,  # pylint: disable=protected-access
-                                         operation=operations_tmpl.format(method_name),
-                                         **merged_kwargs)
+        op_path = operations_tmpl.format(method_name)
 
+        command_name = '{} {}'.format(self.group_name, name) if self.group_name else name
+        command_operation = CommandOperation(
+            command_loader=self.command_loader,
+            op_path=op_path,
+            **merged_kwargs
+        )
+        self.command_loader.add_cli_command(command_name,
+                                            command_operation=command_operation,
+                                            **merged_kwargs)
         return command_name
 
     # pylint: disable=no-self-use
@@ -1288,27 +1320,31 @@ class AzCommandGroup(CommandGroup):
                                setter_name='create_or_update', setter_type=None, setter_arg_name='parameters',
                                child_collection_prop_name=None, child_collection_key='name', child_arg_name='item_name',
                                custom_func_name=None, custom_func_type=None, **kwargs):
-        from azure.cli.core.commands.arm import _cli_generic_update_command
+        from azure.cli.core.commands.command_operation import GenericUpdateCommandOperation
         self._check_stale()
         merged_kwargs = self._flatten_kwargs(kwargs, get_command_type_kwarg())
         merged_kwargs_custom = self._flatten_kwargs(kwargs, get_command_type_kwarg(custom_command=True))
         self._apply_tags(merged_kwargs, kwargs, name)
 
-        getter_op = self._resolve_operation(merged_kwargs, getter_name, getter_type)
-        setter_op = self._resolve_operation(merged_kwargs, setter_name, setter_type)
-        custom_func_op = self._resolve_operation(merged_kwargs_custom, custom_func_name, custom_func_type,
-                                                 custom_command=True) if custom_func_name else None
-        _cli_generic_update_command(
-            self.command_loader,
-            '{} {}'.format(self.group_name, name),
-            getter_op=getter_op,
-            setter_op=setter_op,
+        getter_op_path = self._resolve_operation(merged_kwargs, getter_name, getter_type)
+        setter_op_path = self._resolve_operation(merged_kwargs, setter_name, setter_type)
+        custom_function_op_path = self._resolve_operation(merged_kwargs_custom, custom_func_name, custom_func_type,
+                                                          custom_command=True) if custom_func_name else None
+        command_name = '{} {}'.format(self.group_name, name) if self.group_name else name
+        command_operation = GenericUpdateCommandOperation(
+            command_loader=self.command_loader,
+            getter_op_path=getter_op_path,
+            setter_op_path=setter_op_path,
             setter_arg_name=setter_arg_name,
-            custom_function_op=custom_func_op,
+            custom_function_op_path=custom_function_op_path,
             child_collection_prop_name=child_collection_prop_name,
             child_collection_key=child_collection_key,
             child_arg_name=child_arg_name,
-            **merged_kwargs)
+            **merged_kwargs
+        )
+        self.command_loader.add_cli_command(command_name,
+                                            command_operation=command_operation,
+                                            **merged_kwargs)
 
     def wait_command(self, name, getter_name='get', **kwargs):
         self._wait_command(name, getter_name=getter_name, custom_command=False, **kwargs)
@@ -1320,16 +1356,24 @@ class AzCommandGroup(CommandGroup):
         self._wait_command(name, getter_name=getter_name, getter_type=getter_type, **kwargs)
 
     def _wait_command(self, name, getter_name='get', getter_type=None, custom_command=False, **kwargs):
-        from azure.cli.core.commands.arm import _cli_wait_command
+        from azure.cli.core.commands.command_operation import WaitCommandOperation
         self._check_stale()
         merged_kwargs = self._flatten_kwargs(kwargs, get_command_type_kwarg(custom_command))
         self._apply_tags(merged_kwargs, kwargs, name)
 
         if getter_type:
             merged_kwargs = _merge_kwargs(getter_type.settings, merged_kwargs, CLI_COMMAND_KWARGS)
-        getter_op = self._resolve_operation(merged_kwargs, getter_name, getter_type, custom_command=custom_command)
-        _cli_wait_command(self.command_loader, '{} {}'.format(self.group_name, name), getter_op=getter_op,
-                          custom_command=custom_command, **merged_kwargs)
+        getter_op_path = self._resolve_operation(merged_kwargs, getter_name, getter_type, custom_command=custom_command)
+
+        command_name = '{} {}'.format(self.group_name, name) if self.group_name else name
+        command_operation = WaitCommandOperation(
+            command_loader=self.command_loader,
+            op_path=getter_op_path,
+            **merged_kwargs
+        )
+        self.command_loader.add_cli_command(command_name,
+                                            command_operation=command_operation,
+                                            **merged_kwargs)
 
     def show_command(self, name, getter_name='get', **kwargs):
         self._show_command(name, getter_name=getter_name, custom_command=False, **kwargs)
@@ -1338,16 +1382,24 @@ class AzCommandGroup(CommandGroup):
         self._show_command(name, getter_name=getter_name, custom_command=True, **kwargs)
 
     def _show_command(self, name, getter_name='get', getter_type=None, custom_command=False, **kwargs):
-        from azure.cli.core.commands.arm import _cli_show_command
+        from azure.cli.core.commands.command_operation import ShowCommandOperation
         self._check_stale()
         merged_kwargs = self._flatten_kwargs(kwargs, get_command_type_kwarg(custom_command))
         self._apply_tags(merged_kwargs, kwargs, name)
 
         if getter_type:
             merged_kwargs = _merge_kwargs(getter_type.settings, merged_kwargs, CLI_COMMAND_KWARGS)
-        getter_op = self._resolve_operation(merged_kwargs, getter_name, getter_type, custom_command=custom_command)
-        _cli_show_command(self.command_loader, '{} {}'.format(self.group_name, name), getter_op=getter_op,
-                          custom_command=custom_command, **merged_kwargs)
+        op_path = self._resolve_operation(merged_kwargs, getter_name, getter_type, custom_command=custom_command)
+
+        command_name = '{} {}'.format(self.group_name, name) if self.group_name else name
+        command_operation = ShowCommandOperation(
+            command_loader=self.command_loader,
+            op_path=op_path,
+            **merged_kwargs
+        )
+        self.command_loader.add_cli_command(command_name,
+                                            command_operation=command_operation,
+                                            **merged_kwargs)
 
     def _apply_tags(self, merged_kwargs, kwargs, command_name):
         # don't inherit deprecation or preview info from command group
